#include <unistd.h>
#include <termios.h>
#include <string.h>

#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ioctl.h>

#include <vwm.h>
#include <viper.h>

#include "vwmterm.h"
#include "watchdog.h"
#include "events.h"

gint ptyterm_ON_KEYSTROKE(gint32 keystroke,WINDOW *window)
{
	gchar				*sequence=NULL;
	PTY_TERM			*pty_term;
	
	if(keystroke==KEY_MOUSE) return 1;
	
	viper_thread_enter();
	pty_term=(PTY_TERM*)viper_window_get_userptr(window);
	viper_thread_leave();

	/*	check foreground job control status.  this was moved from the input
		parsing loop on 5-3-2007.  keystrokes occur much less frequently and
		it's really the only time its necessary.	*/
	vwmterm_watchdog(pty_term);

	/*	most applications expect CR not LF	*/	
	if(keystroke==10) keystroke=13;

	if(keystroke==KEY_BACKSPACE) keystroke=pty_term->term_desc.c_cc[VERASE];
	
	/*	some keys require 2-byte
		integers which do not always correspond to the input code/scan code
		expected by the terminal.  In such case we have to do some special
		translation.  This should be rare though.	*/
	if(keystroke==KEY_UP) sequence="\x1B[A";
	if(keystroke==KEY_DOWN) sequence="\x1B[B";
	if(keystroke==KEY_RIGHT) sequence="\x1B[C";
	if(keystroke==KEY_LEFT) sequence="\x1B[D";
	if(keystroke==KEY_F(1)) sequence="\x1B[[A";
	if(keystroke==KEY_F(2)) sequence="\x1B[[B";
	if(keystroke==KEY_F(3)) sequence="\x1B[[C";
	if(keystroke==KEY_F(4)) sequence="\x1B[[D";
	if(keystroke==KEY_F(5)) sequence="\x1B[[E";
	if(keystroke==KEY_F(6)) sequence="\x1B[17~";
	if(keystroke==KEY_F(7)) sequence="\x1B[18~";
	if(keystroke==KEY_F(8)) sequence="\x1B[19~";
	if(keystroke==KEY_F(9)) sequence="\x1B[20~";
	if(keystroke==KEY_F(10)) sequence="\x1B[21~";
	if(keystroke==KEY_F(11)) sequence="\x1B[22~";
	if(keystroke==KEY_F(12)) sequence="\x1B[23~";
	if(keystroke==KEY_HOME) sequence="\x1B[1~";
   if(keystroke==KEY_DC) sequence="\x1B[3~";
	if(keystroke==KEY_END) sequence="\x1B[4~";
	if(keystroke==KEY_NPAGE) sequence="\x1B[6~";
	if(keystroke==KEY_PPAGE) sequence="\x1B[5~";
   
	/*	a simple key was pressed	*/
	if(((keystroke & 0xFFFFFF00)==0) && sequence==NULL)
	{
		write(pty_term->master_fd,(gchar*)&keystroke,sizeof(gchar));
      tcdrain(pty_term->master_fd);
		return 1;
	}

	if(sequence!=NULL)
		write(pty_term->master_fd,sequence,strlen(sequence));
	else
		write(pty_term->master_fd,(gchar*)&keystroke,sizeof(keystroke));
   tcdrain(pty_term->master_fd);

	return 1;
}


gint ptyterm_ON_RESIZE(WINDOW *window,gpointer anything)
{
	PTY_TERM *pty_term;
	
	viper_thread_enter();
	pty_term=(PTY_TERM*)viper_window_get_userptr(window);
	
	getmaxyx(window,pty_term->win_sz.ws_row,pty_term->win_sz.ws_col);
	wresize(pty_term->scrollback,pty_term->win_sz.ws_row,
		pty_term->win_sz.ws_col);
	viper_thread_leave();

	ioctl(pty_term->slave_fd,TIOCSWINSZ,&pty_term->win_sz);
	kill(pty_term->child_pid,SIGWINCH);
		
	return 0;
}

gint ptyterm_ON_DESTROY(WINDOW *window,gpointer anything)
{
	PTY_TERM *pty_term;
	
	pty_term=(PTY_TERM*)anything;
	close(pty_term->master_fd);
	close(pty_term->slave_fd);
   if(pty_term->carryover!=NULL) g_free(pty_term->carryover);
	if(pty_term->scrollback!=NULL) delwin(pty_term->scrollback);
	g_free(pty_term);
	
	return 0;
}

gint ptyterm_ON_CLOSE(WINDOW *window,gpointer anything)
{
	PTY_TERM *pty_term;
	
	pty_term=(PTY_TERM*)anything;
	kill(pty_term->child_pid,SIGKILL);
	waitpid(pty_term->child_pid,NULL,0);
	
	return VIPER_EVENT_WINDOW_DESIST;
}
