/*************************************************************************
 * All portions of code are copyright by their respective author/s.
 * Copyright (C) 2007      Bryan Christ <bryan.christ@hp.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------*/

#include <string.h>

#include <sys/sysinfo.h>
#include <sys/types.h>

#include <viper.h>
#include <pseudo.h>

#include "vwm.h"
#include "vwm_syssaver.h"
#include "vwm_profile.h"

WINDOW*	vwm_fmod_syssaver(gpointer anything)
{
	WINDOW			*window;
   WINDOW   		*statwin;
	gchar				computer[64];
	gchar				*buffer;

	gethostname(computer,sizeof(computer)-1);
   buffer=g_strdup_printf("%s@%s",VWM_LOGIN,computer);

	viper_thread_enter();
	window=viper_window_create("SysSaver",0,0,0.99999,0.99999,FALSE);
	statwin=newwin(8,32,1,1);
   wbkgdset(statwin,VIPER_COLORS(COLOR_WHITE,COLOR_BLACK));
   viper_event_set(window,"window-close",vwm_fmod_syssaver_ON_CLOSE,
      (gpointer)statwin);
   viper_event_set(window,"vwm-clock-tick",vwm_fmod_syssaver_ON_CLOCK_TICK,
      (gpointer)statwin);
   viper_event_set(window,"term-resized",vwm_fmod_syssaver_ON_TERM_RESIZE,
      (gpointer)statwin);
	mvwprintw(statwin,1,1,buffer);
   box(statwin,ACS_VLINE,ACS_HLINE);
	viper_thread_leave();

	g_free(buffer);
	return window;
}

gint vwm_fmod_syssaver_ON_CLOSE(WINDOW *window,gpointer arg)
{
   delwin((WINDOW*)arg);
   return VIPER_EVENT_WINDOW_DESIST;
}

gint vwm_fmod_syssaver_ON_TERM_RESIZE(WINDOW *window,gpointer arg)
{
   /* gint     max_x,max_y; */
   WINDOW   *statwin;         

   statwin=(WINDOW*)arg;
   viper_wresize(window,WSIZE_FULLSCREEN,WSIZE_FULLSCREEN,1);
   /* getmaxyx(SCREEN_WINDOW,max_y,max_x);
   wresize((WINDOW*)arg,max_y-2,max_x-2); */

   return 1;
}
   

gint vwm_fmod_syssaver_ON_CLOCK_TICK(WINDOW *window,gpointer arg) 
{
   WINDOW      	*statwin;
	gdouble			loads[3];
	static gint 	vector_x=1;
   static gint 	vector_y=1;
   static gint 	tick=0;
   gint        	retval;
   /* gint           beg_x,beg_y; */

   tick++;
	/*	move window only every 1/2 second	*/
   if((tick%5)!=0) return 0;

   statwin=(WINDOW*)arg;
   
   retval=window_move_rel(statwin,vector_y,0);
   if(retval==ERR)
	{
		vector_y=0-vector_y;
		window_move_rel(statwin,vector_y,0);
	} 
  
   retval=window_move_rel(statwin,0,vector_x);
	if(retval==ERR)
	{
 		vector_x=0-vector_x;
		window_move_rel(statwin,0,vector_x);
	}

	getloadavg(loads,3);
	mvwprintw(statwin,3,1,"Load: %.2f %.2f %.2f",loads[0],loads[1],loads[2]);
   /* getbegyx(statwin,beg_y,beg_x);
   mvwprintw(statwin,4,1,"x: %02d y:%02d",beg_y,beg_x); */
	werase(window);
   overwrite(statwin,window);
   viper_window_redraw(window);

   return 0;
}
